@@ -139,7 +139,24 @@ impl<T,U> Vecg<T,U> for &[T]
         for &s in self { out.push(m.smult(s)) }      
         out
     }
-
+/*
+    /// Cholesky decomposition of positive definite matrix into LL^T
+    fn cholesky(self) -> Vec<f64> {
+    let n = self.len();
+    let mut res = vec![0.0; n];
+    for i in 0..n {
+        for j in 0..(i+1){
+            let mut s = 0.0;
+            for k in 0..j {
+                s += res[i * n + k] * res[j * n + k];
+            }
+            res[i * n + j] = if i == j { (f64::from(self[i * n + i]) - s).sqrt() } 
+            else { 1.0 / res[j * n + j] * (f64::from(self[i * n + j]) - s) };
+        }
+    }
+    res
+}
+*/
     /// Joint probability density function of two pairwise matched slices 
     fn jointpdf(self,v:&[U]) -> Vec<f64> {     
         let n = self.len();
@@ -169,12 +186,19 @@ impl<T,U> Vecg<T,U> for &[T]
         jpdf.iter().map(|&x| -x*(x.ln()) ).sum() 
     }
 
-    /// Dependence of &[T] &[U] variables.
-    /// e.g. `dependence` returns 0 iff they are statistically 
-    /// component wise independent
+    /// Dependence of &[T] &[U] variables in the range [0,1]
+    /// returns 0 iff they are statistically component wise independent
+    /// returns 1 when they are identical or all their values are unique
     fn dependence(self, v:&[U]) -> f64 {   
-        self.entropy() + v.entropy() - self.jointentropy(v)
+        (self.entropy() + v.entropy())/self.jointentropy(v)-1.0 
     }
+
+    /// Independence of &[T] &[U] variables in the range [1,2]
+    /// returns 2 iff they are statistically component wise independent
+    fn independence(self, v:&[U]) -> f64 {   
+        2.0 * self.jointentropy(v) / (self.entropy() + v.entropy()) 
+    }
+
     /// We define median based correlation as cosine of an angle between two
     /// zero median vectors (analogously to Pearson's zero mean vectors) 
     /// # Example
